@@ -1,7 +1,7 @@
 /*
  * Java-Gnome Bindings Library
  *
- * Copyright 1998-2004 the Java-Gnome Team, all rights reserved.
+ * Copyright 1998-2006 the Java-Gnome Team, all rights reserved.
  *
  * The Java-Gnome bindings library is free software distributed under
  * the terms of the GNU Library General Public License version 2.
@@ -19,24 +19,18 @@ import org.gnu.gtk.event.TreeViewListener;
 import org.gnu.glib.Handle;
 
 /**
- * The TreeView object is a widget for displaying trees and lists. The sole
- * purpose of this widget is for displaying the data on the screen and setting
- * other parts of the view using the associated objects.
+ * TreeView is GTK's Widget for displaying trees and lists. A TreeView itself is
+ * the parent container for <i>presenting</i> the data on screen; other classes
+ * control the layout of that presentation and the mapping from underlying data
+ * to that layout.
  * 
  * <h1>Gtk Tree and List Widgets Overview</h1>
  * <p>
- * The standard tree and list widgets in Gtk are very powerful, but somewhat
- * complex. For your convenience, the Java-Gnome project has derived a number of
- * simpler classes for common uses of these widgets.
- * <dl>
- * <dt>{@link SimpleList}
- * <dd>A single column list of string values.
- * <dt>todo...
- * <dd>todo...
- * </dl>
- * <p>
- * If you choose not to use those, or your requirements are such that you cannot
- * use them, you will have to learn how to use the full objects.
+ * <i>The standard tree and list widgets in GTK+ are very powerful, but
+ * unfortunately are also rather complex and there is only so much that can be
+ * done to mitigate this. If you patiently work though the following
+ * explaination and review the tutorial on the java-gnome website, you should
+ * find yourself quickly up to speed.</i>
  * 
  * <h2>{@link TreeView}</h2>
  * <p>
@@ -55,43 +49,45 @@ import org.gnu.glib.Handle;
  * <p>
  * Models are used to store data. Data is stored in what could be considered a
  * table. There are a number of {@link DataColumn}s, which could be considered
- * as data columns (in fact, in the C version of gtk, they are always referred to
- * as columns; but this can get confused with TreeViewColumns which are quite a
- * different matter). These dataBlocks each store one type of data (String,
- * boolean, int, etc.). The 'rows' of the data table, or the individual records
- * in the data store are accessible using iterators called {@link TreeIter}s.
- * These are used extensively in many methods. Setting data involves getting an
- * iterator (usually be creating a new row) and then setting the value for each
- * of the dataBlocks. The ordering of these dataBlocks has absolutely no meaning -
- * you can decide exactly which blocks are used on screen by setting Attribute
- * mappings to the CellRenderers which render data on the screen (see below).
+ * the columns of that table (in fact, in the C version of GTK, they are always
+ * referred to as columns; but this can get confused with TreeViewColumns which
+ * are quite a different matter). These DataColumns each store one type of data
+ * (String, boolean, int, etc.). The 'rows' of this data table (the individual
+ * records) can be accessed using iterators called {@link TreeIter}s. These are
+ * used extensively in many methods. Setting data involves getting an iterator
+ * (creating a new row gets you a TreeIter) and then setting the value for each
+ * of the DataColumns. The ordering of the DataColumns in the Model has no
+ * meaning. For presentation, you decide exactly which DataColumns are to be
+ * used on screen by mapping them to a a TreeViewColumn via a CellRenderer (see
+ * below).
  * <p>
- * Currently, there are two implementations of {@link TreeModel}
+ * GTK provides two implementations of {@link TreeModel}:
  * <p>
  * {@link ListStore} - This is used for tables and lists. Data is organised in
  * rows and columns.
  * <p>
- * {@link TreeStore} - This is for the hierarchical trees. Data is organised
- * using {@link TreePath}'s.
+ * {@link TreeStore} - This is for data organized in hierarchical trees. You
+ * order your data using {@link TreePath}s.
  * 
  * <h2>{@link TreeViewColumn}</h2>
  * <p>
- * Both trees and lists can have multiple columns of data. Columns are added and
- * removed using the {@link TreeView} class. Columns are objects which determine
+ * Both modes of TreeView (displaying a tree or displaying a list) can have
+ * multiple columns of data. As each TreeViewColumns is set up, you add it to
+ * the parent {@link TreeView} you're creating. The TreeViewColumns determine
  * how the data is displayed. They have settings such as the column title,
  * whether the column can be resized, and even whether the columns can be
  * reorganized (by dragging the columns). Each {@link TreeView} widget has it's
- * own set of columns. Determining how the data is displayed in the columns is
- * done by CellRenderers (see below). Any number of renderers can be added to
- * the same column
+ * own set of TreeViewColumns. Determining how the data is displayed in the
+ * columns is done by CellRenderers (see below). Any number of CellRenderers can
+ * be packed into a given column.
  * 
  * <h2>{@link CellRenderer}</h2>
  * <p>
  * Tree and list `cells' may contain a large variety of data types. Determining
  * how they are displayed is done by the CellRenderer family of classes. If the
  * data is unusual, or you want to combine a number of data types in a single
- * column, you may construct your own renderer. However, you are recommended to
- * stick with the regular choices:
+ * column, you may need to construct your own renderer. However, you are
+ * recommended to stick with the regular choices:
  * <dl>
  * <dt>{@link CellRendererPixbuf}
  * <dd>
@@ -102,17 +98,19 @@ import org.gnu.glib.Handle;
  * buttons.
  * </dl>
  * <p>
- * The CellRenderer's need data to be able to display. This is set using the
- * {@link TreeViewColumn#addAttributeMapping(CellRenderer,
+ * The CellRenderers need to be told what data to display. This is done using
+ * the {@link TreeViewColumn#addAttributeMapping(CellRenderer,
  * CellRendererAttribute, DataColumn)}. The renderer attributes vary with each
  * renderer, for example CellRendererText has a TEXT attribute for the text the
- * be displayed. The final parameter is for the dataBlock in the store in which
+ * be displayed. The final parameter is for the DataColumn in the store in which
  * the data is contained.
- * 
- * 
  * <p>
  * Java-Gnome comes with a number of simple example applications involving
  * trees. They may be useful for learning the functionality of these classes.
+ * <p>
+ * Finally, some people may find use for a convenience wrapper that Java-Gnome
+ * derived; see {@link SimpleList} to display a single column list of string
+ * values.
  * 
  * @author Mark Howard &lt;mh@debian.org&gt;
  */
@@ -211,7 +209,7 @@ public class TreeView extends Container {
     /**
      * Gets the GtkAdjustment currently being used for the horizontal aspect.
      * 
-     * @return A GtkAdjustment object, or NULL if none is currently being used.
+     * @return A GtkAdjustment object, or null if none is currently being used.
      */
     public Adjustment getHAdjustment() {
         Handle hndl = gtk_tree_view_get_hadjustment(getHandle());
@@ -244,7 +242,7 @@ public class TreeView extends Container {
      * Sets the the visibility state of the headers.
      * 
      * @param headersVisible
-     *            TRUE if the headers are visible
+     *            true if the headers are visible
      */
     public void setHeadersVisible(boolean headersVisible) {
         gtk_tree_view_set_headers_visible(getHandle(), headersVisible);
@@ -261,7 +259,7 @@ public class TreeView extends Container {
      * Allow the column title buttons to be clicked.
      * 
      * @param setting
-     *            TRUE if the columns are clickable.
+     *            true if the columns are clickable.
      */
     public void setHeadersClickable(boolean setting) {
         gtk_tree_view_set_headers_clickable(getHandle(), setting);
@@ -280,7 +278,7 @@ public class TreeView extends Container {
      * has lots of columns, generally).
      * 
      * @param setting
-     *            TRUE if the tree requires reading across rows
+     *            true if the tree requires reading across rows
      */
     public void setAlternateRowColor(boolean setting) {
         gtk_tree_view_set_rules_hint(getHandle(), setting);
@@ -304,13 +302,13 @@ public class TreeView extends Container {
     }
 
     /**
-     * Moves column to be after to baseColumn. If baseColumn is NULL, then
+     * Moves column to be after to baseColumn. If baseColumn is null, then
      * column is placed in the first position.
      * 
      * @param column
      *            The GtkTreeViewColumn to be moved.
      * @param baseColumn
-     *            The GtkTreeViewColumn to be moved relative to, or NULL.
+     *            The GtkTreeViewColumn to be moved relative to, or null.
      */
     public void moveColumn(TreeViewColumn column, TreeViewColumn baseColumn) {
         Handle baseColumnHandle = (baseColumn == null ? null : baseColumn
@@ -320,11 +318,11 @@ public class TreeView extends Container {
     }
 
     /**
-     * Sets the column to draw the expander arrow at. If column is NULL, then
+     * Sets the column to draw the expander arrow at. If column is null, then
      * the expander arrow is always at the first visible column.
      * 
      * @param column
-     *            NULL, or the column to draw the expander arrow at.
+     *            null, or the column to draw the expander arrow at.
      */
     public void setExpanderColumn(TreeViewColumn column) {
         Handle expanderColumn = (column == null ? null : column.getHandle());
@@ -405,7 +403,7 @@ public class TreeView extends Container {
      * Moves the alignments of the view to the position specified by path.
      * 
      * @param path
-     *            The path of the row to move to, or NULL.
+     *            The path of the row to move to, or null.
      */
     public void scrollToCell(TreePath path) {
         gtk_tree_view_scroll_to_cell(getHandle(), path.getHandle(), null,
@@ -415,7 +413,7 @@ public class TreeView extends Container {
     /**
      * Sets the current keyboard focus to be at path, and selects it. This is
      * useful when you want to focus the user's attention on a particular row.
-     * If column is specified, and startEditing is TRUE, then editing should be
+     * If column is specified, and startEditing is true, then editing should be
      * started in the specified cell. This function is often followed by
      * {@link Widget#grabFocus()} in order to give keyboard focus to the widget.
      * Please note that editing can only happen when the widget is realized.
@@ -423,9 +421,9 @@ public class TreeView extends Container {
      * @param path
      *            A TreePath
      * @param focusColumn
-     *            A TreeViewColumn, or NULL
+     *            A TreeViewColumn, or null
      * @param startEditing
-     *            TRUE if the specified cell should start being edited.
+     *            true if the specified cell should start being edited.
      */
     public void setCursor(TreePath path, TreeViewColumn focusColumn,
             boolean startEditing) {
@@ -486,7 +484,7 @@ public class TreeView extends Container {
      * @param openAll
      *            Whether to recursively expand, or just expand immediate
      *            children
-     * @return TRUE if the row existed and had children
+     * @return true if the row existed and had children
      */
     public boolean expandRow(TreePath path, boolean openAll) {
         return gtk_tree_view_expand_row(getHandle(), path.getHandle(), openAll);
@@ -497,7 +495,7 @@ public class TreeView extends Container {
      * 
      * @param path
      *            Path to a row in the view
-     * @return TRUE if the row was collapsed.
+     * @return true if the row was collapsed.
      */
     public boolean collapseRow(TreePath path) {
         return gtk_tree_view_collapse_row(getHandle(), path.getHandle());
@@ -508,11 +506,11 @@ public class TreeView extends Container {
      */
 
     /**
-     * Returns TRUE if the node pointed to by path is expanded.
+     * Returns true if the node pointed to by path is expanded.
      * 
      * @param path
      *            A TreePath to test expansion state.
-     * @return TRUE if path is expanded.
+     * @return true if path is expanded.
      */
     public boolean getRowExpanded(TreePath path) {
         return gtk_tree_view_row_expanded(getHandle(), path.getHandle());
@@ -522,7 +520,7 @@ public class TreeView extends Container {
      * This function is a convenience function to allow you to reorder models
      * that support the DragSourceIface and the DragDestIface. Both
      * {@link TreeStore} and {@link ListStore} support these. If reorderable is
-     * TRUE, then the user can reorder the model by dragging and dropping rows.
+     * true, then the user can reorder the model by dragging and dropping rows.
      * The developer can listen to these changes by adding listeners.
      * 
      * <p>
@@ -531,7 +529,7 @@ public class TreeView extends Container {
      * probably handle drag and drop manually.
      * 
      * @param reorderable
-     *            TRUE, if the tree can be reordered.
+     *            true, if the tree can be reordered.
      */
     public void setReorderable(boolean reorderable) {
         gtk_tree_view_set_reorderable(getHandle(), reorderable);
@@ -544,7 +542,7 @@ public class TreeView extends Container {
     /**
      * Sets the model for a GtkTreeView. If the {@link TreeView} already has a
      * model set, it will remove it before setting the new model. If model is
-     * NULL, then it will unset the old model.
+     * null, then it will unset the old model.
      * 
      * @param model
      *            the new model for the {@link TreeView}
@@ -1242,7 +1240,7 @@ public class TreeView extends Container {
     /**
      * Sets the current keyboard focus to be at path, and selects it. This is
      * useful when you want to focus the user's attention on a particular row.
-     * If column is specified, and startEditing is TRUE, then editing should be
+     * If column is specified, and startEditing is true, then editing should be
      * started in the specified cell. If focusCell is also specified, the
      * particular cellRenderer in that column will get focusThis function is
      * often followed by {@link Widget#grabFocus()} in order to give keyboard
@@ -1254,11 +1252,11 @@ public class TreeView extends Container {
      * @param path
      *            A TreePath
      * @param focusColumn
-     *            A TreeViewColumn, or NULL
+     *            A TreeViewColumn, or null
      * @param focusCell
      *            A cellrenderer, or null
      * @param startEditing
-     *            TRUE if the specified cell should start being edited.
+     *            true if the specified cell should start being edited.
      */
     public void setCursor(TreePath path, TreeViewColumn focusColumn,
             CellRenderer focusCell, boolean startEditing) {
@@ -1297,7 +1295,7 @@ public class TreeView extends Container {
      * the tree interactively.
      * 
      * @param enableSearch
-     *            TRUE, if the user can search interactively
+     *            true, if the user can search interactively
      */
     public void setEnableSearch(boolean enableSearch) {
         gtk_tree_view_set_enable_search(getHandle(), enableSearch);
@@ -1364,7 +1362,7 @@ public class TreeView extends Container {
      * are of type {@link TreeViewColumnSizing#FIXED}.
      * 
      * @param enable
-     *            TRUE to enable fixed height mode.
+     *            true to enable fixed height mode.
      */
     public void setFixedHeightMode(boolean enable) {
         gtk_tree_view_set_fixed_height_mode(getHandle(), enable);
@@ -1384,7 +1382,7 @@ public class TreeView extends Container {
      * {@link SelectionMode#BROWSE}.
      * 
      * @param hover
-     *            TRUE to enable hover selection mode.
+     *            true to enable hover selection mode.
      */
     public void setHoverSelection(boolean hover) {
         gtk_tree_view_set_hover_selection(getHandle(), hover);
@@ -1402,7 +1400,7 @@ public class TreeView extends Container {
      * expansion makes rows expand or collapse if the pointer moves over them.
      * 
      * @param expand
-     *            TRUE to enable hover selection mode.
+     *            true to enable hover selection mode.
      */
     public void setHoverExpand(boolean expand) {
         gtk_tree_view_set_hover_expand(getHandle(), expand);
@@ -1461,7 +1459,8 @@ public class TreeView extends Container {
     private Vector listeners = null;
 
     /**
-     * Register an object to handle button events.
+     * Register an object to handle events related to user actions on the
+     * TreeView widget.
      * 
      * @see org.gnu.gtk.event.TreeViewListener
      */
@@ -1485,6 +1484,7 @@ public class TreeView extends Container {
                         TreeViewEvent.Type.START_INTERACTIVE_SEARCH);
                 evtMap.initialize(this, TreeViewEvent.Type.TOGGLE_CURSOR_ROW);
                 evtMap.initialize(this, TreeViewEvent.Type.UNSELECT_ALL);
+                evtMap.initialize(this, TreeViewEvent.Type.MOVE_CURSOR);
                 listeners = new Vector();
             }
             listeners.addElement(listener);
@@ -1515,6 +1515,7 @@ public class TreeView extends Container {
                     TreeViewEvent.Type.START_INTERACTIVE_SEARCH);
             evtMap.uninitialize(this, TreeViewEvent.Type.TOGGLE_CURSOR_ROW);
             evtMap.uninitialize(this, TreeViewEvent.Type.UNSELECT_ALL);
+            evtMap.uninitialize(this, TreeViewEvent.Type.MOVE_CURSOR);
             listeners = null;
         }
     }
@@ -1635,6 +1636,19 @@ public class TreeView extends Container {
         return true;
     }
 
+    /*
+     * Sticking with the stupid parameter names in the GTK API itself.
+     */
+    private boolean handleMoveCursor(int arg1, int arg2) {
+        TreeViewEvent event = new TreeViewEvent(this,
+                TreeViewEvent.Type.MOVE_CURSOR);
+        MovementStep whichOne = MovementStep.intern(arg1);
+        event.setMovementStep(whichOne);
+        event.setHowMany(arg2);
+        fireTreeViewEvent(event);
+        return true;
+    }
+
     public Class getEventListenerClass(String signal) {
         Class cls = evtMap.getEventListenerClass(signal);
         if (cls == null)
@@ -1687,6 +1701,8 @@ public class TreeView extends Container {
                 "handleStartInteractiveSearch",
                 TreeViewEvent.Type.START_INTERACTIVE_SEARCH,
                 TreeViewListener.class);
+        anEvtMap.addEvent("move-cursor", "handleMoveCursor",
+                TreeViewEvent.Type.MOVE_CURSOR, TreeViewListener.class);
     }
 
     native static final protected int gtk_tree_view_get_type();
