@@ -10,8 +10,10 @@
 #include <jni.h>
 #include <string.h>
 #include <gdk/gdk.h>
-#include "jg_jnu.h"
+#include <cairo.h>
+#include "gtk_java.h"
 
+#include "org_gnu_gdk_Pixbuf.h"
 #ifdef __cplusplus
 extern "C" 
 {
@@ -121,48 +123,34 @@ extern "C"
 //    gdk_pixbuf_render_pixmap_and_mask (pixbuf_g, pixmapReturn_g, maskReturn_g, 
 //            alphaThreshold_g);
 //}
-//
-///*
-// * Class:     org.gnu.gdk.Pixbuf
-// * Method:    gdk_pixbuf_get_from_drawable
-// */
-//JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1drawable (JNIEnv *env, 
-//    jclass cls, jobject dest, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
-//    jint width, jint height) 
-//{
-//    GdkPixbuf *dest_g = (GdkPixbuf *)getPointerFromHandle(env, dest);
-//    GdkDrawable *src_g = (GdkDrawable *)getPointerFromHandle(env, src);
-//    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
-//    gint32 srcX_g = (gint32) srcX;
-//    gint32 srcY_g = (gint32) srcY;
-//    gint32 destX_g = (gint32) destX;
-//    gint32 destY_g = (gint32) destY;
-//    gint32 width_g = (gint32) width;
-//    gint32 height_g = (gint32) height;
-//    return getHandleFromPointer(env, gdk_pixbuf_get_from_drawable (dest_g, src_g, cmap_g, srcX_g, srcY_g, 
-//            destX_g, destY_g, width_g, height_g));
-//}
-//
-///*
-// * Class:     org.gnu.gdk.Pixbuf
-// * Method:    gdk_pixbuf_get_from_image
-// */
-//JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1image (JNIEnv *env, 
-//    jclass cls, jobject dest, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
-//    jint width, jint height) 
-//{
-//    GdkPixbuf *dest_g = (GdkPixbuf *)getPointerFromHandle(env, dest);
-//    GdkImage *src_g = (GdkImage *)getPointerFromHandle(env, src);
-//    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
-//    gint32 srcX_g = (gint32) srcX;
-//    gint32 srcY_g = (gint32) srcY;
-//    gint32 destX_g = (gint32) destX;
-//    gint32 destY_g = (gint32) destY;
-//    gint32 width_g = (gint32) width;
-//    gint32 height_g = (gint32) height;
-//    return getHandleFromPointer(env, gdk_pixbuf_get_from_image (dest_g, src_g, cmap_g, srcX_g, srcY_g, destX_g, 
-//            destY_g, width_g, height_g));
-//}
+
+/*
+ * Class:     org.gnu.gdk.Pixbuf
+ * Method:    gdk_pixbuf_get_from_drawable
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1drawable (JNIEnv *env, 
+    jclass cls, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
+    jint width, jint height) 
+{
+    GdkDrawable *src_g = (GdkDrawable *)getPointerFromHandle(env, src);
+    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_get_from_drawable (NULL,
+    		src_g, cmap_g, srcX, srcY, destX, destY, width, height));
+}
+
+/*
+ * Class:     org.gnu.gdk.Pixbuf
+ * Method:    gdk_pixbuf_get_from_image
+ */
+JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1from_1image (JNIEnv *env, 
+    jclass cls, jobject src, jobject cmap, jint srcX, jint srcY, jint destX, jint destY, 
+    jint width, jint height) 
+{
+    GdkImage *src_g = (GdkImage *)getPointerFromHandle(env, src);
+    GdkColormap *cmap_g = (GdkColormap *)getPointerFromHandle(env, cmap);
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_get_from_image (NULL,
+    		src_g, cmap_g, srcX, srcY, destX, destY, width, height));
+}
 
 /*
  * Class:     org.gnu.gdk.Pixbuf
@@ -216,11 +204,11 @@ JNIEXPORT jbyteArray JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1pixels (J
     jclass cls, jobject pixbuf) 
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    gchar *result_g = gdk_pixbuf_get_pixels (pixbuf_g);
-    jsize result_len = result_g ? strlen (result_g) : 0;
+    guchar *result_g = gdk_pixbuf_get_pixels (pixbuf_g);
+    jsize result_len = result_g ? strlen ((gchar*)result_g) : 0;
     jbyteArray result_j = result_g ? (*env)->NewByteArray (env, result_len) : NULL;
     if (result_g) 
-    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, result_g);
+    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, (jbyte*)result_g);
     return result_j;
 }
 
@@ -269,8 +257,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new (JNIEnv *env,
     gint32 bitsPerSample_g = (gint32) bitsPerSample;
     gint32 width_g = (gint32) width;
     gint32 height_g = (gint32) height;
-    return getHandleFromPointer(env, gdk_pixbuf_new (colorspace_g, hasAlpha_g, bitsPerSample_g, width_g, 
-            height_g));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_new (colorspace_g,
+    		hasAlpha_g, bitsPerSample_g, width_g, height_g));
 }
 
 /*
@@ -280,7 +268,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new (JNIEnv *env,
 JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1copy (JNIEnv *env, jclass cls, jobject pixbuf) 
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    return getHandleFromPointer(env, gdk_pixbuf_copy (pixbuf_g));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_copy (pixbuf_g));
 }
 
 /*
@@ -295,7 +283,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1subpixbuf (J
     gint32 srcY_g = (gint32) srcY;
     gint32 width_g = (gint32) width;
     gint32 height_g = (gint32) height;
-    return getHandleFromPointer(env, 
+    return getGObjectHandle(env, (GObject *)
     		gdk_pixbuf_new_subpixbuf (srcPixbuf_g, srcX_g, srcY_g, width_g, height_g));
 }
 
@@ -308,10 +296,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file (
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getHandleFromPointer(env, gdk_pixbuf_new_from_file (filename_g, &error_g));
+    jobject result =  getGObjectHandle(env, (GObject *)
+    		gdk_pixbuf_new_from_file (filename_g, &error_g));
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
-    if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    if (NULL != error_g){
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
+    }
     return result;
 }
 
@@ -324,10 +314,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file_1
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getHandleFromPointer(env, gdk_pixbuf_new_from_file_at_size (filename_g, width, height, &error_g));
+    GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_size (filename_g, width,
+    		height, &error_g);
+    jobject result =  getGObjectHandle(env, (GObject *) pixbuf);
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
     return result;
 }
 
@@ -340,10 +332,12 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1file_1
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
     GError* error_g = NULL;
-    jobject result =  getHandleFromPointer(env, gdk_pixbuf_new_from_file_at_scale (filename_g, width, height, aspect, &error_g));
+    jobject result =  getGObjectHandle(env, (GObject *)
+    		gdk_pixbuf_new_from_file_at_scale (filename_g, width, height, aspect,
+    		&error_g));
     (*env)->ReleaseStringUTFChars(env, filename, filename_g);
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
     return result;
 }
 
@@ -355,12 +349,13 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1file_1info
   (JNIEnv *env, jclass cls, jstring filename, jintArray width, jintArray height)
 {
     const gchar* filename_g = (*env)->GetStringUTFChars(env, filename, 0);
-	gint* w = (gint*)(*env)->GetIntArrayElements(env, width, NULL);
-	gint* h = (gint*)(*env)->GetIntArrayElements(env, height, NULL);
-	jobject handle = getHandleFromPointer(env, gdk_pixbuf_get_file_info(filename_g, w, h));
-	(*env)->ReleaseIntArrayElements(env, width, (jint*)w, 0);
-	(*env)->ReleaseIntArrayElements(env, height, (jint*)h, 0);
-	return handle;
+    gint* w = (gint*)(*env)->GetIntArrayElements(env, width, NULL);
+    gint* h = (gint*)(*env)->GetIntArrayElements(env, height, NULL);
+    // GdkPixbufFormat is owned by the GdkPixbuf.
+    jobject handle = getStructHandle(env, gdk_pixbuf_get_file_info(filename_g, w, h), NULL, NULL);
+    (*env)->ReleaseIntArrayElements(env, width, (jint*)w, 0);
+    (*env)->ReleaseIntArrayElements(env, height, (jint*)h, 0);
+    return handle;
 }
 
 
@@ -376,7 +371,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1xpm_1d
     const gchar** data_g = (const gchar**)g_malloc(data_len + 1);
     (*env)->GetByteArrayRegion(env, data, 0, data_len, (jbyte*)data_g);
     data_g[data_len] = 0;
-    return getHandleFromPointer(env, gdk_pixbuf_new_from_xpm_data (data_g));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_new_from_xpm_data (data_g));
 }
 
 /*
@@ -388,15 +383,15 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1inline
 {
     gint32 dataLength_g = (gint32) dataLength;
     jint data_len = (*env)->GetArrayLength(env, data);
-    gchar* data_g = (gchar*)g_malloc(data_len + 1);
+    guchar* data_g = (guchar*)g_malloc(data_len + 1);
     gboolean copyPixels_g = (gboolean) copyPixels;
     GError* error_g = NULL;
     (*env)->GetByteArrayRegion(env, data, 0, data_len, (jbyte*)data_g);
     data_g[data_len] = 0;
-    jobject handle = getHandleFromPointer(env, 
+    jobject handle = getGObjectHandle(env, (GObject *)
     	gdk_pixbuf_new_from_inline (dataLength_g, data_g, copyPixels_g, &error_g));
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
     return handle;
 }
 
@@ -405,7 +400,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1new_1from_1inline
  * Method:    gdk_pixbuf_savev
  */
 JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1savev
-  (JNIEnv *env, jclass cls, jobject pixbuf, jstring filename, jstring type, jobjectArray keys, jobjectArray values, jobject error)
+  (JNIEnv *env, jclass cls, jobject pixbuf, jstring filename, jstring type,
+  		jobjectArray keys, jobjectArray values, jobject error)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
     const gchar* f = (*env)->GetStringUTFChars(env, filename, NULL);
@@ -415,7 +411,7 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1savev
     GError* error_g = NULL;
 	jboolean val = gdk_pixbuf_savev(pixbuf_g, f, t, k, v, &error_g);
     if (NULL != error_g)
-    	updateHandle(env, error, error_g);
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
 	(*env)->ReleaseStringUTFChars(env, filename, f);
 	(*env)->ReleaseStringUTFChars(env, type, t);
 	freeStringArray(env, keys, k);
@@ -427,24 +423,32 @@ JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1savev
  * Class:     org_gnu_gdk_Pixbuf
  * Method:    gdk_pixbuf_save_to_bufferv
  */
-JNIEXPORT jboolean JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1save_1to_1bufferv
-  (JNIEnv *env, jclass cls, jobject pixbuf, jstring buffer, jstring type, jobjectArray keys, jobjectArray values, jobject error)
+JNIEXPORT jbyteArray JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1save_1to_1bufferv
+  (JNIEnv *env, jclass cls, jobject pixbuf, jstring type,
+  		jobjectArray keys, jobjectArray values, jobject error)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    gchar* b = (gchar*)(*env)->GetStringUTFChars(env, buffer, NULL);
+    gchar* b = NULL;
     const gchar* t = (*env)->GetStringUTFChars(env, type, NULL);
     gchar** k = getStringArray(env, keys);
     gchar** v = getStringArray(env, values);
     GError* error_g = NULL;
     gsize size;
-	jboolean val = gdk_pixbuf_save_to_bufferv(pixbuf_g, &b, &size, t, k, v, &error_g);
-    if (NULL != error_g)
-    	updateHandle(env, error, error_g);
-	(*env)->ReleaseStringUTFChars(env, buffer, b);
+    jbyteArray ret_val;
+    ret_val = NULL;
+    gdk_pixbuf_save_to_bufferv(pixbuf_g, &b, &size, t, k, v, &error_g);
+    if (error_g != NULL)
+    	updateStructHandle(env, error, error_g, (JGFreeFunc) g_error_free);
+    else	{
+    	ret_val = (*env)->NewByteArray(env, (jsize) size);
+    	(*env)->SetByteArrayRegion(env, ret_val, 0, (jsize) size, (jbyte *) b); 
+    	
+    }
 	(*env)->ReleaseStringUTFChars(env, type, t);
 	freeStringArray(env, keys, k);
 	freeStringArray(env, values, v);
-	return val;
+	
+	return ret_val;
 }
 
 /*
@@ -455,7 +459,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1rotate_1simple
   (JNIEnv *env, jclass cls, jobject pixbuf, jint direction)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    return getHandleFromPointer(env, gdk_pixbuf_rotate_simple(pixbuf_g, (GdkPixbufRotation)direction));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_rotate_simple(pixbuf_g,
+    		(GdkPixbufRotation) direction));
 }
 
 /*
@@ -466,7 +471,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1flip
   (JNIEnv *env, jclass cls, jobject pixbuf, jboolean horizontal)
 {
     GdkPixbuf *pixbuf_g = (GdkPixbuf *)getPointerFromHandle(env, pixbuf);
-    return getHandleFromPointer(env, gdk_pixbuf_flip(pixbuf_g, (gboolean)horizontal));
+    return getGObjectHandle(env, (GObject *) gdk_pixbuf_flip(pixbuf_g,
+    		(gboolean)horizontal));
 }  
 
 /*
@@ -493,7 +499,8 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1add_1alpha (JNIEn
     gint32 r_g = (gint32) r;
     gint32 g_g = (gint32) g;
     gint32 b_g = (gint32) b;
-    return getHandleFromPointer(env,
+    // returned pixbuf has ref count of 1.  should we use a different getGObjectHandle function here?
+    return getGObjectHandle(env, (GObject *)
     		gdk_pixbuf_add_alpha (pixbuf_g, substituteColor_g, r_g, g_g, b_g));
 }
 
@@ -621,7 +628,7 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1scale_1simple (JN
     gint32 destWidth_g = (gint32) destWidth;
     gint32 destHeight_g = (gint32) destHeight;
     GdkInterpType interpType_g = (GdkInterpType) interpType;
-    return getHandleFromPointer(env, 
+    return getGObjectHandle(env, (GObject *)
     		gdk_pixbuf_scale_simple (src_g, destWidth_g, destHeight_g, interpType_g));
 }
 
@@ -641,8 +648,9 @@ JNIEXPORT jobject JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1composite_1color_
     gint32 checkSize_g = (gint32) checkSize;
     gint32 color1_g = (gint32) color1;
     gint32 color2_g = (gint32) color2;
-    return getHandleFromPointer(env, gdk_pixbuf_composite_color_simple (src_g, destWidth_g, destHeight_g, 
-            interpType_g, overallAlpha_g, checkSize_g, color1_g, color2_g));
+    return getGObjectHandle(env, G_OBJECT(gdk_pixbuf_composite_color_simple
+    		(src_g, destWidth_g, destHeight_g, interpType_g, overallAlpha_g,
+    		checkSize_g, color1_g, color2_g)));
 }
 
 /*
@@ -661,10 +669,20 @@ JNIEXPORT jbyteArray JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1option (J
     jsize result_len = result_g ? strlen (result_g) : 0;
     jbyteArray result_j = result_g ? (*env)->NewByteArray (env, result_len) : NULL;
     if (result_g) 
-    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, result_g);
+    	(*env)->SetByteArrayRegion (env, result_j, 0, result_len, (jbyte*)result_g);
     return result_j;
 }
 
+/*
+ * Class:     org.gnu.gdk.Pixbuf
+ * Method:    gdk_pixbuf_get_type
+ */
+JNIEXPORT jint JNICALL Java_org_gnu_gdk_Pixbuf_gdk_1pixbuf_1get_1type (JNIEnv *env, 
+    jclass cls)
+{
+	return (jint)gdk_pixbuf_get_type();
+}
+
 #ifdef __cplusplus
 }
 
