@@ -1,7 +1,7 @@
 /*
  * Java-Gnome Bindings Library
  *
- * Copyright 1998-2004 the Java-Gnome Team, all rights reserved.
+ * Copyright 1998-2006 the Java-Gnome Team, all rights reserved.
  *
  * The Java-Gnome bindings library is free software distributed under
  * the terms of the GNU Library General Public License version 2.
@@ -124,7 +124,12 @@ JNIEXPORT jint JNICALL Java_org_gnu_glib_Type_get_1INT
 JNIEXPORT jint JNICALL Java_org_gnu_glib_Type_get_1LONG
     (JNIEnv *env, jclass cls) 
 {
-	return (jint)G_TYPE_LONG;
+	/*
+	 * Note we use signed 64 bit wide integer values as 
+	 * matches the Java platforms definiton for the long
+	 * datatype
+	 */
+	return (jint)G_TYPE_INT64;
 }
 
 /*
