@@ -114,6 +114,31 @@ JNIEXPORT void JNICALL Java_org_gnu_gtk_Calendar_gtk_1calendar_1get_1date (JNIEn
     (*env)->ReleaseIntArrayElements (env, day, (jint *) day_g, 0);
 }
 
+/*
+ * Class:     org.gnu.gtk.Calendar
+ * Method:    getYearMonthDay
+ */
+JNIEXPORT void JNICALL Java_org_gnu_gtk_Calendar_getYearMonthDay(JNIEnv *env, jclass 
+    cls, jobject calendar, jintArray triple) 
+{
+    jint *triple_c;
+    triple_c = (*env)->GetIntArrayElements(env, triple, NULL);
+    if (triple_c == NULL) {
+	    return;
+    }
+
+    GtkCalendar *calendar_g = (GtkCalendar *)getPointerFromHandle(env, calendar);
+    guint year_g, month_g, day_g;
+
+    gtk_calendar_get_date (calendar_g, &year_g, &month_g, &day_g);
+
+    triple_c[0] = year_g;
+    triple_c[1] = month_g;
+    triple_c[2] = day_g;
+
+    (*env)->ReleaseIntArrayElements (env, triple, triple_c, 0);
+}
+
 /*
  * Class:     org.gnu.gtk.Calendar
  * Method:    gtk_calendar_freeze
